/**
 * \file            user_config.h
 * \brief           user configuration of the application
 */

/*
 * Copyright (c) 2022 Dino Haw
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * This file is part of mOTA - The Over-The-Air technology component for MCU.
 *
 * Author:          Dino Haw <347341799@qq.com>
 * Version:         v1.0.2
 * Change Logs:
 * Date           Author       Notes
 * 2022-11-23     Dino         the first version
 * 2022-12-04     Dino         1. һ¼汾Ļƣѡд APP 
 *                             2. ӳָ̼ѡ
 *                             3.  flash Ĳ user_config.h 
 *                             4. ǷжϹ̼Сѡ
 * 2022-12-07     Dino          ONCHIP_FLASH_ONCE_WRITE_BYTE 
 */

/**
 * OTAĿѡ
 */
#include "app_config.h"

/**
 * ѡ
 * : 
 *    APP:      еĹ̼
 *    download: ڸ¹̼ʱĹ̼ʱ
 *    factory:  ڴſڽ»ָ̼ʹõ
 *    ONE_PART_PROJECT:      APP 
 *    DOUBLE_PART_PROJECT:  ˫ APP + download 
 *    TRIPLE_PART_PROJECT:   APP + download + factory 
 * ѡ: 
 *    ONE_PART_PROJECT       0
 *    DOUBLE_PART_PROJECT    1
 *    TRIPLE_PART_PROJECT    2
 */
#define USING_PART_PROJECT                  TRIPLE_PART_PROJECT


/**
 * flash
 * ˵: 
 *    øĴС
 * ע: 
 *    Ƭ Flash ҳ룬 ׵ַ Flash  ÿ page  sector ׵ַ̼޷У
 *     SPI Flash ķҪСдȵΪλж룬 sector Ϊλж룡
 */
#define ONCHIP_FLASH_SIZE                   ((uint32_t)(512 * 1024))    /* Ƭ flash λ: byte */
#define BOOTLOADER_SIZE                     ((uint32_t)(32 * 1024))     /* Ԥ bootloader Ŀռ䣬λ: byteСҪڱ̱ĴС */
#define APP_PART_SIZE                       ((uint32_t)(32 * 1024))     /* Ԥ APP Ŀռ䣬λ: byteעҳ룩 */
#define DOWNLOAD_PART_SIZE                  (APP_PART_SIZE)             /* Ԥ download Ŀռ䣬λ: byteעҳ룬ʹʱд0 */
#define FACTORY_PART_SIZE                   (APP_PART_SIZE)             /* Ԥ factory Ŀռ䣬λ: byteעҳ룬ʹʱд0 */


/**
 * ѡǷý
 * ˵: 
 *    ̼мܣá̼޼ܣɰѡǷ
 *    * AES256_KEY  32 ֽڣ AES256_IV  16 ֽ
 * ѡ: 
 *    0: 
 *    1: 
 */
#define ENABLE_DECRYPT                      1
    #if (ENABLE_DECRYPT)
    #define AES256_KEY                      "0123456789ABCDEF0123456789ABCDEF"  /*  32 ֽ */
    #define AES256_IV                       "0123456789ABCDEF"                  /*  16 ֽ */
    #endif


/**
 * Ƭ flash һдСֽ
 * ˵
 *    ÿ MCU ڲ flash οдֽͬṩѡãλ byte
 *    ֵĴСڲ flash ĲӿھĬ 4 byte ãҪѡζʲô
 */
#define ONCHIP_FLASH_ONCE_WRITE_BYTE        4


/**
 * ѡ bootloader ǷҪ̼µķ
 * ˵
 *    豸ϵʱ bootloader  bootloader ҪһȡǷҪй̼µķ
 * ͣ
 *    USING_HOST_CMD_UPDATE:     ָͨƽ̼ģʽ˷ʽȴһʱ䣬ȷǷҪй̼£
 *                               ȴʱͨ WAIT_HOST_DATA_MAX_TIME 
 *    USING_APP_SET_FLAG_UPDATE: ͨ APP ñ־λ bootloader ʱͨȡñ־λжҪй̼
 *                               ̼
 * ѡ
 *    USING_HOST_CMD_UPDATE       0
 *    USING_APP_SET_FLAG_UPDATE   1
 */
#define USING_IS_NEED_UPDATE_PROJECT        USING_APP_SET_FLAG_UPDATE
    #if (USING_IS_NEED_UPDATE_PROJECT == USING_HOST_CMD_UPDATE)
    #define WAIT_HOST_DATA_MAX_TIME         (3 * 1000)      /* õȴݵȴʱ䣬λ ms */
    #else
    #define WAIT_HOST_DATA_MAX_TIME         (60 * 1000)     /* õȴݵȴʱ䣬λ ms
                                                             * ʱûյָζܵߣ
                                                             * ʱ bootloader ᳢ת APP
                                                             * Ϊ 0 ʾߣҲת APP
                                                             */
    #define FIRMWARE_UPDATE_VAR_ADDR        0x20000000      /* ̼±־λĴŵַ޸ģһҪ APP һ£ */
    #define FIRMWARE_UPDATE_MAGIC_WORD      0xA5A5A5A5      /* ̼Ҫµǣ޸ģһҪ APP һ£ */
    #define FIRMWARE_RECOVERY_MAGIC_WORD    0x5A5A5A5A      /* Ҫָ̼ǣ޸ģһҪ APP һ£ */
    #define BOOTLOADER_RESET_MAGIC_WORD     0xAAAAAAAA      /* bootloader λǣ޸ģһҪ APP һ£ */
    #endif


/**
 * ѡǷʹðָ̼ѡ
 * ˵: 
 *    1. ʹñѡǰѡЧǰȷ˰ factory пõĹ̼
 *    2. ѡʹðָ̼ʱҪðţʹõİͱͬҪԼúͳʼ GPIO
 *    3. ѡ豸 bootloader ʱָ̼ͨ豸 APP ѡ޷ЧģҪ APP Ҳͬ
 *    4. ѡָָ̼ͨķʽͻͬʱʹãҲԲñѡ
 *    5.  factory  factory ޿ù̼ʱǿлָ̼ᴥ FACTORY_NO_FIRMWARE_SOLUTION ѡ
 * ͣ
 *    ENABLE_FACTORY_FIRMWARE_BUTTON: ѡǷóָ̼
 *    FACTORY_FIRMWARE_BUTTON_PRESS:  ʱĵƽ߼
 *    FACTORY_FIRMWARE_BUTTON_TIME:   ĳʱ䣬λ ms
 * ѡ
 *    ENABLE_FACTORY_FIRMWARE_BUTTON ѡ: 
 *        0: 
 *        1: 
 *    FACTORY_FIRMWARE_BUTTON_PRESS ѡ: 
 *        KEY_PRESS_LOW:  ʾʱΪ͵ƽ
 *        KEY_PRESS_HIGH: ʾʱΪߵƽ
 *    FACTORY_FIRMWARE_BUTTON_TIME ѡ: 
 *        ĳʱ䣬λ ms ܴ 65535
 */ 
#if (USING_PART_PROJECT == TRIPLE_PART_PROJECT)
#define ENABLE_FACTORY_FIRMWARE_BUTTON      0
#define FACTORY_FIRMWARE_BUTTON_PRESS       KEY_PRESS_LOW
#define FACTORY_FIRMWARE_BUTTON_TIME        3000
#endif


/**
 * ѡжϹ̼Ƿ񳬹Сѡ
 * ˵: 
 *    жʱ⴦޸Ĵ룬λ app.c  APP_Running   EXE_FLOW_VERIFY_FIRMWARE_HEAD 
 * ѡ: 
 *    0: 
 *    1: 
 */ 
#define ENABLE_CHECK_FIRMWARE_SIZE          1


/**
 * ѡԶ¹̼Ĵ
 * ˵: 
 *    1. ڹ̼¹豸쳣ϵѡǷԶغõĹ̼ԼԶµĴ
 *    2. ѡִȼλµķʽζųλʱδݣ򽫻ִλĹ̼
 * : 
 *    
 *    DO_NOT_AUTO_UPDATE:           ҪԶ£ϣֶϵָ̼Ļ
 *    ERASE_DOWNLOAD_PART_PROJECT:  ɺ download 豸ϵʱͨ download ޿ù̼
 *                                  жǷҪԶ¹̼
 *                                  * ѡ˷󣬽޷ѡǷϵ APP Ĺ̼аУ( USING_APP_SAFETY_CHECK_PROJECT )
 *                                    Ϊ APP ̼УԴ download Ĺ̼
 *    MODIFY_DOWNLOAD_PART_PROJECT: ɺ޸ download Ĺ̼ͷİ汾Ϣ豸ϵʱԱ download ̼ͷ¼¾ɰ汾
 *                                  ¾ɰ汾һ£ʼԶ¹̼
 *                                  * ַʽҪ޸ download ݣƣ
 *                                    1. ƣϵʱУ APP Ĺ̼ȷԺԣ APP ̼𻵻۸ʱİȫԣ
 *                                             Խָ̼Чϵͳİȫȼ
 *                                    2. ƣҪԱͷڵ flash sector д룬ζÿθ¶ͬ sector Ρ
 *                                              flash СȱȽϴ󣬸÷ʽ޷ʵֵģΪ MCU  flash С
 *                                             ϴ STM32  F4 ϵУ sector0  16 byte  VERSION_WRITE_TO_APP ѡ
 *                                             ʵڵòʧ sector С 1  2 kByte Ƽѡ VERSION_WRITE_TO_APP                                   
 *    VERSION_WRITE_TO_APP:         ɺµĹ̼汾д APP βռ 16 byte 豸ϵʱԱ download ̼ͷ
 *                                  ¼İ汾 APP ŵİ汾汾һ£ʼԶ¹̼
 *                                  * ַʽƣ
 *                                    1. ƣϵʱУ APP Ĺ̼ȷԺԣ APP ̼𻵻۸ʱİȫԣ
 *                                             Խָ̼Чϵͳİȫȼ
 *                                    2. ƣҪռ APP  16 byte Ŀռ
 * ѡ: 
 *    DO_NOT_AUTO_UPDATE             0
 *    ERASE_DOWNLOAD_PART_PROJECT    1
 *    MODIFY_DOWNLOAD_PART_PROJECT   2
 *    VERSION_WRITE_TO_APP           3
 */
#if (USING_PART_PROJECT > ONE_PART_PROJECT)
#define USING_AUTO_UPDATE_PROJECT           MODIFY_DOWNLOAD_PART_PROJECT
#endif


/**
 * Ƭ Flash ù̼ sector Ĳ ( TODO: δʵ )
 * ˵: 
 *    USING_AUTO_UPDATE_PROJECT = MODIFY_DOWNLOAD_PART_PROJECT ʱҪ̼ sector Ĳȣλ byte
 */
#if (USING_AUTO_UPDATE_PROJECT == MODIFY_DOWNLOAD_PART_PROJECT)
#define ONCHIP_FLASH_ERASE_GRANULARITY      FLASH_PAGE_SIZE
#endif


/**
 * ѡǷϵ APP Ĺ̼аУ  APP ̼ʱĲ
 * ˵: 
 *    1. USING_AUTO_UPDATE_PROJECT = MODIFY_DOWNLOAD_PART_PROJECT ʱòŻЧ
 *    2. ֲƷԹ̼аȼҪ󣬱֧ APP ̼Լ飬ͨѡǷ
 *    3. ʱͨ USING_APP_SAFETY_PROJECT ѡ APP ̼ʱĲ
 * : 
 *    DO_NOT_CHECK      : У APP ̼
 *    CHECK_UNLESS_EMPTY: У APP ̼޷УʱУ飬ȷ APP ڵ bootloader С APP ̼У󣬽
 *                        ԶѿúȷĹ̼ APP 
 *                        * ޷ APP ̼Уǣ download  factory ޿ù̼
 *    AUTO_UPDATE_APP   : APP ̼УԶúȷĹ̼ APP
 *                        * Ҫעǣѡ˱ѡζʮ APP ԣҲ˵ APP ̼У鲻ͨ
 *                          ޷Уʱ download  factory ޿ù̼ͣ bootloader Уת APP 
 *                           APP ڹ̼
 *                        * Ҫرǣһᵼ APP ޷ִУǱͨ¼̼¼ MCU  flash У
 *                          ΪʱͨĹ̼³ִУ download  factory ޿ù̼ APP ̼޷
 *                          У飬Ĺ̼̣ bootloader ̼£ѡ CHECK_UNLESS_EMPTY ѡ
 *                        * ˴Զº USING_AUTO_UPDATE_PROJECT Ĳͬѡ APP ̼У鲻ͨʱŻ
 *                          Զ£ USING_AUTO_UPDATE_PROJECT ӱѡй̼Զ 
 *    DO_NOT_DO_ANYTHING: APP̼УҪκβͣ bootloader ɣ download  factory пõĹ̼
 *                        * Ҫעǣ DO_NOT_DO_ANYTHING ѡֹ APP Ϊʱ USING_AUTO_UPDATE_PROJECT 
 *                          Զµ DO_NOT_DO_ANYTHING ֻAPPΪУ鲻ͨҪֹԶ£Ҫ޸
 *                          Ϸ USING_AUTO_UPDATE_PROJECT ѡΪ DO_NOT_AUTO_UPDATE
 * ѡ:  USING_APP_SAFETY_PROJECT
 *    DO_NOT_CHECK           0
 *    CHECK_UNLESS_EMPTY     1
 *    AUTO_UPDATE_APP        2
 *    DO_NOT_DO_ANYTHING     3
 */
#if (USING_PART_PROJECT > ONE_PART_PROJECT &&   \
     (USING_AUTO_UPDATE_PROJECT == MODIFY_DOWNLOAD_PART_PROJECT || USING_AUTO_UPDATE_PROJECT == VERSION_WRITE_TO_APP))
#define USING_APP_SAFETY_CHECK_PROJECT      CHECK_UNLESS_EMPTY
#endif


/**
 * ѡǷʹ factory Ĺ̼
 * ˵: 
 *    Զ»У APP ̼ʱ APP ̼ã download ûпõĹ̼ʱ factory 
 *     factory пõĹ̼ѡǷ factory Ĺ̼ APP 
 * ѡ: 
 *    0: ʹ
 *    1: ʹ
 */
#if (USING_PART_PROJECT == TRIPLE_PART_PROJECT && \
     (USING_APP_SAFETY_CHECK_PROJECT == CHECK_UNLESS_EMPTY || USING_APP_SAFETY_CHECK_PROJECT == AUTO_UPDATE_APP))
#define ENABLE_USE_FACTORY_FIRWARE          1
#endif


/**
 * ѡҪָ̼ʱ factory ޹̼̼УʱĽ
 * ͣ
 *    JUMP_TO_APP:           ת APP
 *    WAIT_FOR_NEW_FIRMWARE: ȴλµĹ̼
 * ѡ
 *    JUMP_TO_APP             0
 *    WAIT_FOR_NEW_FIRMWARE   1
 */
#if (USING_PART_PROJECT == TRIPLE_PART_PROJECT)
#define FACTORY_NO_FIRMWARE_SOLUTION        JUMP_TO_APP          
#endif


/**
 * ѡǷԶ̼ķ
 * ˵: 
 *    ѡΪΪĽдһ̶ȱ֤̼ȴʩ
 *    £۱Ƿãָ̼ᱻΪ APP ʹ
 *    £ú󣬹ָ̼Ϊ APP ʱᱻΪ download ʹ
 *    £ãָ̼Ϊ APP ʱᱨΪʧ
 * ѡ: 
 *    0: 
 *    1: 
 */
#if (USING_PART_PROJECT > ONE_PART_PROJECT)
#define ENABLE_AUTO_CORRECT_PART             1
#endif


/**
 * ѡǷʹ SPI Flash Ź̼ ( TODO: δʵ )
 * ˵: 
 *    SPI Flash ڷùֻ̼ download  factory 
 *    ʹ SFUD ⣬˿ԶݲͬҺͲͬ SPI Flash 
 *    SPI Flash ĵײӿֲļ sfud_port.c ʵڲ API 
 *    ʹõ SPI Flash ֧ SFDP Ҫ޸ sfdu_flash_def.h ļ
 *    ޸ġֲʽ͸ݣ:  https://github.com/armink/SFUD
 * ѡ: 
 *    0: 
 *    1: 
 */
#if (USING_PART_PROJECT > ONE_PART_PROJECT)
#define ENABLE_SPI_FLASH                    0
#endif


/**
 * SPI Flash ù̼ sector Ĳ ( TODO: δʵ )
 * ˵: 
 *    USING_AUTO_UPDATE_PROJECT = MODIFY_DOWNLOAD_PART_PROJECT   SPI Flash ʱҪ̼ sector Ĳȣ
 *    λ byte
 */
#if (ENABLE_SPI_FLASH && USING_AUTO_UPDATE_PROJECT == MODIFY_DOWNLOAD_PART_PROJECT)
#define SPI_FLASH_ERASE_GRANULARITY         4096
#endif


/**
 * ѡĴλ ( TODO: ʱ֧Ƭ)
 * ˵: 
 *    ѡ SPI Flash Ź̼ʱҪָ download  factory Ĵλ
 *    ʹ FAL ⣬ڹ flash ķ SPI Flash ʱ
 *    ݣ:  https://github.com/RT-Thread-packages/fal
 * : 
 *    STORE_IN_ONCHIP_FLASH : Ƭ Flash
 *    STORE_IN_SPI_FLASH    : SPI Flash
 * ע: 
 *    زǿƽ APP  SPI Flash ڡ򣬹̼޷У
 * ѡ: 
 *    STORE_IN_ONCHIP_FLASH  0
 *    STORE_IN_SPI_FLASH     1
 */
#if (ENABLE_SPI_FLASH && USING_PART_PROJECT > ONE_PART_PROJECT)
#define DOWNLOAD_PART_LOCATION              STORE_IN_ONCHIP_FLASH
#define FACTORY_PART_LOCATION               STORE_IN_ONCHIP_FLASH
#endif




