/**
 * \file            bsp_flash.c
 * \brief           onchip flash driver
 */

/*
 * Copyright (c) 2022 Dino Haw
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * This file is part of mOTA - The Over-The-Air technology component for MCU.
 *
 * Author:          Dino Haw <347341799@qq.com>
 * Version:         v1.0.0
 */

/* Includes ------------------------------------------------------------------*/
#include "bsp_flash.h"


#if (ENABLE_SPI_FLASH == 0)
/* Extern functions -----------------------------------------------------------*/
extern int read(long offset, uint8_t *buf, size_t size);
extern int write(long offset, const uint8_t *buf, size_t size);
extern int erase(long offset, size_t size);


/* Private variables ---------------------------------------------------------*/
static struct BSP_FLASH *_part_head;


/* Private function prototypes -----------------------------------------------*/
static void _Flash_Add(struct BSP_FLASH *part);


/* Exported functions ---------------------------------------------------------*/
/**
 * @brief  ڲ flash ʼ
 * @note   
 * @param[in]  part: flash 
 * @param[in]  name: flash 
 * @param[in]  addr: flash ʼַ
 * @param[in]  size: flash Сλ byte
 * @retval None
 */
void BSP_Flash_Init(struct BSP_FLASH *part, const char *name, uint32_t addr, uint32_t size)
{
    ASSERT(part != NULL);
    
    memcpy(part->name, name, MAX_NAME_LEN);
    part->addr = addr;
    part->len  = size;
    
    _Flash_Add(part);
}


/**
 * @brief  ȡڲ flash
 * @note   
 * @param[in]   part: flash 
 * @param[in]   relative_addr: ȡԵַ
 * @param[out]  buff: ȡŵݳ
 * @param[in]   size: ҪȡĴСλ byte
 * @retval -1: ʧܡ -2: ָΪա 0 ֵ: Ѷȡݳȣλ byte
 */
int BSP_Flash_Read(const struct BSP_FLASH *part, uint32_t relative_addr, uint8_t *buff, uint32_t size)
{
    ASSERT(part != NULL);
    
    return read(part->addr + relative_addr, buff, size);
}


/**
 * @brief  дڲ flash
 * @note    buff  flash Сд byte ᷢ
 * @param[in]  part: flash 
 * @param[in]  relative_addr: дԵַ
 * @param[in]  buff: Ҫд
 * @param[in]  size: ҪдݴСλ byte
 * @retval -1: ʧܡ -2: ָΪա 0 ֵ: дݳȣλ byte
 */
inline int BSP_Flash_Write(const struct BSP_FLASH *part, uint32_t relative_addr, const uint8_t *buff, uint32_t size)
{
    ASSERT(part != NULL);
    
    return write(part->addr + relative_addr, buff, size);
}


/**
 * @brief  ڲ flash
 * @note   
 * @param[in]  part: flash 
 * @param[in]  relative_addr: ʼַ
 * @param[in]  size: ҪĴСλ: byte
 * @retval -1: ʧܡ -2: ָΪա 0 ֵ: Ѳݳȣλ: byte
 */
inline int BSP_Flash_Erase(const struct BSP_FLASH *part, uint32_t relative_addr, uint32_t size)
{
    ASSERT(part != NULL);
    
    return erase(part->addr + relative_addr, size);
}


/**
 * @brief  ͨ flash ȡ flash 
 * @note   
 * @param[in]  part_name: flash 
 * @retval flash 
 */
struct BSP_FLASH *BSP_Flash_GetHandle(const char *part_name)
{
    struct BSP_FLASH *part;
    
    for (part = _part_head; part != NULL; part = part->next)
    {
        if (strncmp(part->name, part_name, MAX_NAME_LEN) == 0)
            return part;
    }
    
    return NULL;
}


/* Private functions ---------------------------------------------------------*/
/**
 * @brief   flash ӽ
 * @note   
 * @param[in]  part: flash 
 * @retval None
 */
static void _Flash_Add(struct BSP_FLASH *part)
{
    struct BSP_FLASH *target;

    for (target = _part_head; target != NULL; target = target->next)
    {
        if (target == part)
            return;
    }

    part->next = _part_head;
    _part_head  = part;
}

#endif


