/**
 * \file            bsp_key.h
 * \brief           universal key driver
 */

/*
 * Copyright (c) 2022 Dino Haw
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * This file is part of mOTA - The Over-The-Air technology component for MCU.
 *
 * Author:          Dino Haw <347341799@qq.com>
 * Version:         v1.0.0
 */

#ifndef __BSP_KEY_H__
#define __BSP_KEY_H__

#include "bsp_common.h"

#define KEY_TEST                            0

/*  */
#define KEY_CLICK_EVENT_WHEN_DBLCLICK       0       /* 趨˫ʱǷ񻹴¼1˫ʱɴ¼0˫ʱ¼ᴥ˫¼ */
#define KEY_DBLCLICK_PRESS_VALID            0       /* ˫ʱѡڶΰʱ¼ǵڶΰʱſŴ¼ */
                                                    /* 1ڶΰʱ¼0ڶΰ²ſŴ¼ */
#define KEY_LONG_PRESS_TIME                 1200    /* ʱ䣬λms */
#define KEY_DBLCLICK_TIME                   200     /* ˫ʱ䡣λms */

/* ǳ */
#define KEY_USER_DEFINE_PRESS_TIME          0       /* ûÿ趨ͬĳʱ䣬Ϊ 0 ʱʾȫĳʱʹ KEY_LONG_PRESS_TIME  */
                                                    /* Ϊ 1 ʱʾû뵥Ϊÿó¼Ĵʱ */
#define KEY_LONG_PRESS_CONTINU_TRIG_TIME    0       /* һֱ KEY_LONG_PRESS_TIME λʱ󣬴˲ʾû°ÿһʱ䴥һ KEY_LONG_PRESS_HOLD ¼ */
                                                    /* Ϊ 0 ʱʾûһֱ°೤ʱ䣬ֻһ KEY_LONG_PRESS ¼ֱûſΪֹܼ¸¼Ĳ */
#define KEY_CONSECUTIVE_READ_TIME           4       /* ȡֵ˲ʵַʽİֵ8 */
                                                    /* ֵӰȡռֵʹ¼ʱ */
                                                    /* ĵ¼ʱ = KEY_CONSECUTIVE_READ_TIME * BSP_Key_HandlerĲֵλms */
                                                    /* ¼ʱ */

enum KEY_LEVEL
{
    KEY_PRESS_LOW  = 0x00,
    KEY_PRESS_HIGH = 0xFF
};

enum KEY_STATE
{
    KEY_UP = 0,
    KEY_DOWN
};

enum KEY_EVENT
{
    KEY_PRESS           = 0,        /* հ£ÿ¼ᴥ¼ǷҪ¼ûѡǷע¼Ļص */
    KEY_CLICK           = 1,        /*  */
    KEY_DBLCLICK        = 2,        /* ˫ */
    KEY_LONG_PRESS      = 3,        /*  */
    KEY_LONG_PRESS_HOLD = 4,        /*  */
    KEY_EVENT_NUM,                  /* ¼ */
    KEY_NONE_EVENT                  /* Ĭ¼ */
};


typedef void (*Key_EventCallback)(uint8_t id, enum KEY_EVENT  event);


struct KEY_STRUCT
{
    enum KEY_EVENT  event;          /* ¼¼ */
    enum KEY_LEVEL  press_level;    /* ¼°ʲôƽ߻ͣ */
    enum KEY_STATE  key_state;      /* ¼ǰ״̬» */
    
    uint8_t id;                     /* ṩûİ idЩû뽫еİصдһӿڣ˾ٷûʶΪĸ¼ */
                                    /* Ϳɼһӿں֧256 id */
    uint8_t state;                  /* ״̬ */
    uint8_t continuous_value;       /* 棬һ8λȣڴȡһͬʱ̵İ״̬8 */
    
    uint16_t tick;                  /* ڼʱʱ */
#if (KEY_USER_DEFINE_PRESS_TIME)
    uint16_t press_time;            /* ʱ */
#endif
    
    uint8_t (*GetState)(void);                      /* ƽ״̬ȡҪûʵ */
    Key_EventCallback  Callback[ KEY_EVENT_NUM ];   /* صɰ¼Ҫûʵ */
    
    struct KEY_STRUCT *next;        /* 򵥵ĵ */
};

#if (KEY_USER_DEFINE_PRESS_TIME)
int8_t BSP_Key_Init(struct KEY_STRUCT *key, uint8_t id, uint8_t (*KEY_GetState)(void), enum KEY_LEVEL press_level, uint16_t press_time);
#else
int8_t BSP_Key_Init(struct KEY_STRUCT *key, uint8_t id, uint8_t (*KEY_GetState)(void), enum KEY_LEVEL press_level);
#endif
int8_t BSP_Key_Start(struct KEY_STRUCT *key);
void   BSP_Key_Register(struct KEY_STRUCT *key, enum KEY_EVENT event, Key_EventCallback callback);
void   BSP_Key_Handler(uint8_t ms);

#if (KEY_TEST)
void Key_Test(void);
#endif

#endif

